"""medication administrations urls."""

from django.urls import path

from dfhir.medicationadministrations.views import (
    MedicationAdministrationDetailView,
    MedicationAdministrationListView,
)

app_name = "medicationadministrations"

urlpatterns = [
    path(
        "medicationadministrations/",
        MedicationAdministrationListView.as_view(),
        name="medicationadministration-list",
    ),
    path(
        "medicationadministrations/<int:pk>/",
        MedicationAdministrationDetailView.as_view(),
        name="medicationadministration-detail",
    ),
]



"""activity definition models."""

from django.db import models

from dfhir.activitydefinitions.choices import (
    ActivityDefinitionIntentChoices,
    ActivityDefinitionKindChoices,
    ActivityDefinitionParticipantTypeChoices,
    ActivityDefinitionPriorityChoices,
    ActivityDefinitionStatus,
)
from dfhir.base.models import BaseReference, Identifier, TimeStampedModel


class ActivityDefinitionParticipantTypeReference(BaseReference):
    """activity definition participant type reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference_identifier",
    )
    biologically_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_biologically_derived_product_reference_biologically_derived_product",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_practitioner_type_reference_care_plan",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_device_reference_idevice",
    )
    device_definition = models.ForeignKey(
        "devicedefinitions.DeviceDefinition",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_device_definition_reference_device_definition",
    )
    endpoint = models.ForeignKey(
        "endpoints.Endpoint",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference_endpoint",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference_group",
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference_healthcare_service",
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference_location",
    )
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference_medication",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference_organization",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference_patient",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_practitioner_type_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_practitioner_type_reference_practitioner_role",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference_related_person",
    )
    specimen = models.ForeignKey(
        "specimens.Specimen",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference_specimen",
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference_substance",
    )


class ActivityDefinitionParticipant(TimeStampedModel):
    """activity definition participant model."""

    type = models.CharField(
        max_length=255,
        choices=ActivityDefinitionParticipantTypeChoices.choices,
        null=True,
    )
    # TODO: type_canonical = models.ForeignKey(
    #     "CapabilityStatementCanonical",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="activity_definition_participant",
    # )
    type_reference = models.ForeignKey(
        ActivityDefinitionParticipantTypeReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_type_reference",
    )
    role = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_role",
    )
    function = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_participant_function",
    )


class ActivityDefinitionProductProductReference(BaseReference):
    """activity definition product reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_product_product_reference_identifier",
    )
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_product_product_reference_medication",
    )
    # TODO: ingredient = models.ForeignKey(
    #     "ingredients.Ingredient",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="activity_definition_product_reference_ingredient",
    # )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_product_product_reference_substance",
    )
    # TODO: substance_definition = models.ForeignKey(
    #     "substancedefinitions.SubstanceDefinition",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="activity_definition_product_product_reference_substance_definition",
    # )


class ActivityDefinitionSubjectReference(BaseReference):
    """activity definition subject reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_subject_reference_identifier",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_subject_reference_group",
    )
    # TODO: medicinal_product_definition = models.ForeignKey(
    #     "medicinalproductdefinitions.MedicinalProductDefinition",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="activity_definition_subject_reference_medicinal_product_definition",
    # )

    # TODO: substance_definition = models.ForeignKey(
    #     "substancedefinitions.SubstanceDefinition",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="activity_definition_subject_reference_substance_definition",
    # )

    # TODO: administrable_product_definition = models.ForeignKey(
    #     "administrableproductdefinitions.AdministrableProductDefinition",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="activity_definition_administrable_product_definition_reference_identifier",
    # )

    # TODO: manufactured_item_definition = models.ForeignKey(
    #     "manufactureditemdefinitions.ManufacturedItemDefinition",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="activity_definition_manufactured_item_definition_reference_identifier",
    # )

    # TODO: packaged_product_definition = models.ForeignKey(
    #     "packagedproductdefinitions.PackagedProductDefinition",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="activity_definition_packaged_product_definition_reference_identifier",
    # )


class ActivityDefinitionDynamicValue(TimeStampedModel):
    """activity definition dynamic value model."""

    path = models.CharField(max_length=255, null=True)
    expression = models.ForeignKey(
        "base.Expression",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_dynamic_value_expression",
    )


class ActivityDefinition(TimeStampedModel):
    """activity definition model."""

    url = models.URLField(null=True)
    identifier = models.ManyToManyField(
        Identifier, related_name="activity_definition_identifier", blank=True
    )
    version = models.CharField(max_length=255, null=True)
    version_algorithm_string = models.CharField(max_length=255, null=True)
    version_algorithm_coding = models.ForeignKey(
        "base.Coding",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_version_algorithm_coding",
        null=True,
    )
    name = models.CharField(max_length=255, null=True)
    title = models.CharField(max_length=255, null=True)
    subtitle = models.CharField(max_length=255, null=True)
    status = models.CharField(
        max_length=255, null=True, choices=ActivityDefinitionStatus.choices
    )
    experimental = models.BooleanField(null=True)
    subject_codealbe_concept = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_subject_codeable_concept",
        null=True,
    )
    subject_reference = models.ForeignKey(
        ActivityDefinitionSubjectReference,
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_subject_reference",
        null=True,
    )
    # TODO: subject_canonical = models.ForeignKey(
    #     "EvidenceVariable",
    #     on_delete=models.DO_NOTHING,
    #     related_name="activity_definition_subject_canonical",
    #     null=True,
    # )
    date = models.DateTimeField(null=True)
    publisher = models.CharField(max_length=255, null=True)
    contact = models.ManyToManyField(
        "base.ContactDetail", related_name="activity_definition_contact", blank=True
    )
    description = models.TextField(null=True)
    use_context = models.ManyToManyField(
        "base.UsageContext", related_name="activity_definition_use_context", blank=True
    )
    jurisdiction = models.ManyToManyField(
        "base.CodeableConcept",
        related_name="activity_definition_jurisdiction",
        blank=True,
    )
    purpose = models.TextField(null=True)
    usage = models.TextField(null=True)
    copyright = models.TextField(null=True)
    copyright_label = models.TextField(null=True)
    approval_date = models.DateField(null=True)
    last_review_date = models.DateField(null=True)
    effective_period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_period",
        null=True,
    )
    topic = models.ManyToManyField(
        "base.CodeableConcept", related_name="activity_definition_topic", blank=True
    )
    author = models.ManyToManyField(
        "base.ContactDetail", related_name="activity_definition_author", blank=True
    )
    editor = models.ManyToManyField(
        "base.ContactDetail", related_name="activity_definition_editor", blank=True
    )
    reviewer = models.ManyToManyField(
        "base.ContactDetail", related_name="activity_definition_reviewer", blank=True
    )
    endorser = models.ManyToManyField(
        "base.ContactDetail", related_name="activity_definition_endorser", blank=True
    )
    related_artifact = models.ManyToManyField(
        "base.RelatedArtifact",
        related_name="activity_definition_related_artifact",
        blank=True,
    )

    # TODO: library = models.ManyToManyField(
    #     "CanonicalLibrary", related_name="activity_definition_library", blank=True
    # )
    kind = models.CharField(
        max_length=255, null=True, choices=ActivityDefinitionKindChoices.choices
    )
    # TODO: profile = models.ForeignKey(
    #     "StructuredDefinitionCanonical",
    #     on_delete=models.DO_NOTHING,
    #     related_name="activity_definition_profile",
    #     null=True,
    # )
    code = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_code",
        null=True,
    )
    intent = models.CharField(
        max_length=255, null=True, choices=ActivityDefinitionIntentChoices.choices
    )
    priority = models.CharField(
        max_length=255, null=True, choices=ActivityDefinitionPriorityChoices.choices
    )
    do_not_perform = models.BooleanField(null=True)
    timing_timing = models.ForeignKey(
        "base.Timing",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_timing_timing",
        null=True,
    )
    timing_age = models.ForeignKey(
        "base.Age",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_timing_age",
        null=True,
    )
    timing_range = models.ForeignKey(
        "base.Range",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_timing_range",
        null=True,
    )
    timing_duration = models.ForeignKey(
        "base.Duration",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_timing_duration",
        null=True,
    )
    as_needed_boolean = models.BooleanField(null=True)
    as_needed_codeable_concept = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_as_needed_codeable_concept",
        null=True,
    )
    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_location",
        null=True,
    )
    participant = models.ManyToManyField(
        ActivityDefinitionParticipant,
        related_name="activity_definition_participant",
        blank=True,
    )
    product_reference = models.ForeignKey(
        ActivityDefinitionProductProductReference,
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_product_reference",
        null=True,
    )
    product_codeable_concept = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_product_codeable_concept",
        null=True,
    )
    quantity = models.ForeignKey(
        "base.SimpleQuantity",
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_quantity",
        null=True,
    )
    # TODO: dosage = models.ManyToManyField(
    #     "base.Dosage",
    #     related_name="activity_definition_dosage",
    #     blank=True,
    # )
    body_site = models.ManyToManyField(
        "base.CodeableConcept", related_name="activity_definition_body_site", blank=True
    )
    # TODO: subject_requirements = models.ForeignKey(
    #     "SpecimenDefinitionCanonical",
    #     on_delete=models.DO_NOTHING,
    #     related_name="activity_definition_subject_reference",
    #     null=True,
    # )

    # TODO: observation_requirement = models.ForeignKey(
    #     "ObservationDefinitionCanonical",
    #     on_delete=models.DO_NOTHING,
    #     related_name="activity_definition_observation_requirement",
    #     null=True,
    # )

    # TODO: observation_result_requirement = models.ForeignKey(
    #     "ObservationDefinitionCanonical",
    #     on_delete=models.DO_NOTHING,
    #     related_name="activity_definition_observation_result_requirement",
    #     null=True,
    # )

    # TODO: transform = models.ForeignKey(
    #     "StructureMapCanonical",
    #     on_delete=models.DO_NOTHING,
    #     related_name="activity_definition_transform",
    #     null=True,
    # )
    dynamic_value = models.ManyToManyField(
        ActivityDefinitionDynamicValue,
        related_name="activity_definition_note",
        blank=True,
    )


class ActivityDefinitionReference(BaseReference):
    """ActivityDefinition reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_definition_reference_identifier",
    )
    activity_definition = models.ForeignKey(
        ActivityDefinition,
        on_delete=models.DO_NOTHING,
        related_name="activity_definition_reference",
        null=True,
    )


class ActivityDefinitionPlanDefinitionReference(BaseReference):
    """activity plan definition reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_plan_definition_reference_identifier",
    )
    activity_definition = models.ForeignKey(
        ActivityDefinition,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_plan_definition_reference_activity_definition",
    )
    plan_definition = models.ForeignKey(
        "plandefinitions.PlanDefinition",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="activity_plan_definition_reference_plan_definition",
    )



"""Contract models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    Annotation,
    Attachment,
    BaseReference,
    CodeableConcept,
    Coding,
    Identifier,
    Money,
    OrganizationReference,
    Period,
    Quantity,
    Reference,
    Signature,
    SimpleQuantity,
    TimeStampedModel,
    Timing,
)

from . import choices


class ContractAuthorReference(BaseReference):
    """Contract Author Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_author_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="contract_author_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="contract_author_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="contract_author_reference_practitioner_role",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="contract_author_reference_organization",
        null=True,
    )


class ContractSubjectReference(BaseReference):
    """Subject reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="subject_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="subject_reference_patient",
        null=True,
    )


class ContractContentDefinitionPublisherReference(BaseReference):
    """Contract Content Definition Publisher Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_content_definition_publisher_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="contract_content_definition_publisher_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="contract_content_definition_publisher_reference_practitioner_role",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="contract_content_definition_publisher_reference_organization",
        null=True,
    )


class ContractContentDefinition(TimeStampedModel):
    """Contract Content Definition model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_content_definition_type",
        null=True,
    )
    sub_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_content_definition_sub_type",
        null=True,
    )
    publisher = models.ForeignKey(
        ContractContentDefinitionPublisherReference,
        on_delete=models.CASCADE,
        related_name="contract_content_definition_publisher",
        null=True,
    )
    publication_date = models.DateTimeField(null=True)
    publication_status = models.CharField(
        max_length=255,
        null=True,
        choices=choices.ContractContentDefinitionPublicationStatusChoices.choices,
    )
    copyright = models.TextField(null=True)


class ContractTermSecurityLabel(TimeStampedModel):
    """Contract Term Security Label model."""

    number = ArrayField(models.IntegerField(), null=True)
    classification = models.ForeignKey(
        Coding,
        on_delete=models.CASCADE,
        related_name="contract_term_security_label_classification",
        null=True,
    )
    category = models.ManyToManyField(
        Coding, related_name="contract_term_security_label_category", blank=True
    )
    control = models.ManyToManyField(
        Coding, related_name="contract_term_security_label_control", blank=True
    )


class ContractTermOfferPartyReferenceReference(BaseReference):
    """Contract Term Offer Party Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_term_offer_party_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="contract_term_offer_party_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="contract_term_offer_party_reference_related_person",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="contract_term_offer_party_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="contract_term_offer_party_reference_practitioner_role",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="contract_term_offer_party_reference_device",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.CASCADE,
        related_name="contract_term_offer_party_reference_group",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="contract_term_offer_party_reference_organization",
        null=True,
    )


class ContractTermOfferParty(TimeStampedModel):
    """Contract Term Offer Party model."""

    reference = models.ManyToManyField(
        ContractTermOfferPartyReferenceReference,
        related_name="contract_term_offer_party_reference",
        blank=True,
    )
    role = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_offer_party_role",
        null=True,
    )


class ContractTermOfferAnswer(TimeStampedModel):
    """Contract Term Offer Answer model."""

    value_boolean = models.BooleanField(null=True)
    value_decimal = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    value_integer = models.IntegerField(null=True)
    value_date = models.DateField(null=True)
    value_date_time = models.DateTimeField(null=True)
    value_time = models.TimeField(null=True)
    value_string = models.CharField(max_length=255, null=True)
    value_uri = models.URLField(null=True)
    value_attachment = models.ForeignKey(
        Attachment,
        on_delete=models.CASCADE,
        related_name="contract_term_offer_answer_attachment",
        null=True,
    )
    value_coding = models.ForeignKey(
        Coding,
        on_delete=models.CASCADE,
        related_name="contract_term_offer_answer_coding",
        null=True,
    )
    value_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.CASCADE,
        related_name="contract_term_offer_answer_quantity",
        null=True,
    )
    value_reference = models.ForeignKey(
        Reference,
        on_delete=models.CASCADE,
        related_name="contract_term_offer_answer_reference",
        null=True,
    )


class ContractTermOffer(TimeStampedModel):
    """Contract Term Offer model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="contract_term_offer_identifier",
        blank=True,
    )
    party = models.ManyToManyField(
        ContractTermOfferParty,
        related_name="contract_term_offer_party",
        blank=True,
    )
    topic = models.ForeignKey(
        Reference,
        on_delete=models.CASCADE,
        related_name="contract_term_offer_topic",
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_offer_type",
        null=True,
    )
    decision = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_offer_decision",
        null=True,
    )
    decision_mode = models.ManyToManyField(
        CodeableConcept,
        related_name="contract_term_offer_decision_mode",
        blank=True,
    )
    answer = models.ManyToManyField(
        ContractTermOfferAnswer,
        related_name="contract_term_offer_answer",
        blank=True,
    )
    text = models.TextField(null=True)
    link_id = ArrayField(models.CharField(max_length=255), null=True)
    security_label_number = ArrayField(models.IntegerField(), null=True)


class ContractTermAssetContext(TimeStampedModel):
    """Contract Term Asset Context model."""

    reference = models.ForeignKey(
        Reference,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_context_reference",
        null=True,
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_context_code",
        null=True,
    )
    text = models.TextField(null=True)


class ContractTermAssetValuedItemResponsibleReference(BaseReference):
    """Contract Term ASet Valued item responsible model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="contract_tem_asset_valued_item_responsible_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        related_name="contract_tem_asset_valued_item_responsible_organization",
        on_delete=models.CASCADE,
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        related_name="contract_tem_asset_valued_item_responsible_patient",
        on_delete=models.CASCADE,
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        related_name="contract_tem_asset_valued_item_responsible_practitioner",
        on_delete=models.CASCADE,
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        related_name="contract_tem_asset_valued_item_responsible_practitioner_role",
        on_delete=models.CASCADE,
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        related_name="contract_tem_asset_valued_item_responsible_related_person",
        on_delete=models.CASCADE,
        null=True,
    )


class ContractTermAssetValuedItemRecipientReference(BaseReference):
    """Contract Term Asset Valued Item Recipient model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="contract_tem_asset_valued_item_recipient_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        related_name="contract_tem_asset_valued_item_recipient_organization",
        on_delete=models.CASCADE,
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        related_name="contract_tem_asset_valued_item_recipient_patient",
        on_delete=models.CASCADE,
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        related_name="contract_tem_asset_valued_item_recipient_practitioner",
        on_delete=models.CASCADE,
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        related_name="contract_tem_asset_valued_item_recipient_practitioner_role",
        on_delete=models.CASCADE,
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        related_name="contract_tem_asset_valued_item_recipient_related_person",
        on_delete=models.CASCADE,
        null=True,
    )


class ContractTermAssetValuedItem(TimeStampedModel):
    """Contract Term Asset Valued Item model."""

    entity_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_valued_item_entity_codeable_concept",
        null=True,
    )
    entity_reference = models.ForeignKey(
        Reference,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_valued_item_entity_reference",
        null=True,
    )
    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_valued_item_identifier",
        null=True,
    )
    effective_time = models.DateTimeField(null=True)
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_valued_item_quantity",
        null=True,
    )
    unit_price = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_valued_item_unit_price",
        null=True,
    )
    factor = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    points = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    net = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_valued_item_net",
        null=True,
    )
    payment = models.CharField(max_length=255, null=True)
    payment_date = models.DateTimeField(null=True)
    responsible = models.ForeignKey(
        ContractTermAssetValuedItemResponsibleReference,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_valued_item_responsible",
        null=True,
    )
    recipient = models.ForeignKey(
        ContractTermAssetValuedItemRecipientReference,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_valued_item_recipient",
        null=True,
    )
    link_id = ArrayField(models.CharField(max_length=255), null=True)
    security_label_number = ArrayField(models.IntegerField(), null=True)


class ContractTermAsset(TimeStampedModel):
    """Contract Term Asset model."""

    scope = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_scope",
        null=True,
    )
    type = models.ManyToManyField(
        CodeableConcept,
        related_name="contract_term_asset_type",
        blank=True,
    )
    type_reference = models.ManyToManyField(
        Reference,
        related_name="contract_term_asset_type_reference",
        blank=True,
    )
    subtype = models.ManyToManyField(
        CodeableConcept,
        related_name="contract_term_asset_subtype",
        blank=True,
    )
    relationship = models.ForeignKey(
        Coding,
        on_delete=models.CASCADE,
        related_name="contract_term_asset_relationship",
        null=True,
    )

    context = models.ManyToManyField(
        ContractTermAssetContext,
        related_name="contract_term_asset_context",
        blank=True,
    )
    condition = models.TextField(null=True)
    period_type = models.ManyToManyField(
        CodeableConcept,
        related_name="contract_term_asset_period_type",
        blank=True,
    )
    period = models.ManyToManyField(
        Period,
        related_name="contract_term_asset_period",
        blank=True,
    )
    use_period = models.ManyToManyField(
        Period,
        related_name="contract_term_asset_use_period",
        blank=True,
    )
    text = models.TextField(null=True)
    link_id = ArrayField(models.CharField(max_length=255), null=True)
    answer = models.ManyToManyField(
        ContractTermOfferAnswer,
        related_name="contract_term_asset_answer",
        blank=True,
    )
    security_label_number = ArrayField(models.IntegerField(), null=True)
    valued_item = models.ManyToManyField(
        ContractTermAssetValuedItem,
        related_name="contract_term_asset_valued_item",
        blank=True,
    )


class ContractTermActionSubjectReferenceReference(BaseReference):
    """Contract Term Action Subject Reference Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_term_action_subject_reference_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="contract_term_action_subject_reference_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="contract_term_action_subject_reference_reference_related_person",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="contract_term_action_subject_reference_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="contract_term_action_subject_reference_reference_practitioner_role",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="contract_term_action_subject_reference_reference_device",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.CASCADE,
        related_name="contract_term_action_subject_reference_reference_group",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="contract_term_action_subject_reference_reference_organization",
        null=True,
    )


class ContractTermActionSubject(TimeStampedModel):
    """Contract Term Action Subject model."""

    reference = models.ManyToManyField(
        ContractTermActionSubjectReferenceReference,
        related_name="contract_term_action_subject_reference",
        blank=True,
    )
    role = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_action_subject_role",
        null=True,
    )


class ContractTermActionRequesterReference(BaseReference):
    """Contract Term Action Requester Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_term_action_requester_reference_identifier",
        null=True,
    )

    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="contract_term_action_requester_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="contract_term_action_requester_reference_related_person",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="contract_term_action_requester_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="contract_term_action_requester_reference_practitioner_role",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="contract_term_action_requester_reference_device",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.CASCADE,
        related_name="contract_term_action_requester_reference_group",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="contract_term_action_requester_reference_organization",
        null=True,
    )


class ContractTermActionPerformerReference(BaseReference):
    """Contract Term Action Performer Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer_reference_identifier",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer_reference_related_person",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer_reference_practitioner_role",
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer_reference_care_team",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer_reference_device",
        null=True,
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer_reference_substance",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer_reference_organization",
        null=True,
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer_reference_location",
        null=True,
    )


class ContractTermActionReasonReference(BaseReference):
    """Contract Term Action Reason Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_term_action_reason_reference_identifier",
        null=True,
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.CASCADE,
        related_name="contract_term_action_reason_reference_condition",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.CASCADE,
        related_name="contract_term_action_reason_reference_observation",
        null=True,
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.CASCADE,
        related_name="contract_term_action_reason_reference_diagnostic_report",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.CASCADE,
        related_name="contract_term_action_reason_reference_document_reference",
        null=True,
    )
    # questionnaire = models.ForeignKey(
    #     "questionnaires.Questionnaire",
    #     on_delete=models.CASCADE,
    #     related_name="contract_term_action_reason_reference_questionnaire",
    #     null=True,
    # )
    # questionnaire_response = models.ForeignKey(
    #     "questionnaireresponses.QuestionnaireResponse",
    #     on_delete=models.CASCADE,
    #     related_name="contract_term_action_reason_reference_questionnaire_response",
    #     null=True,
    # )


class ContractTermActionReasonCodeableReference(TimeStampedModel):
    """Contract term action reason codeable reference model."""

    reference = models.ForeignKey(
        ContractTermActionReasonReference,
        on_delete=models.CASCADE,
        related_name="contract_term_action_reason_codeable_reference_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_action_reason_codeable_reference_concept",
        null=True,
    )


class ContractTermAction(TimeStampedModel):
    """Contract Term Action model."""

    do_not_perform = models.BooleanField(default=False)
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_action_type",
        null=True,
    )
    subject = models.ManyToManyField(
        ContractTermActionSubject,
        related_name="contract_term_action_subject",
        blank=True,
    )
    intent = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_action_intent",
        null=True,
    )
    link_id = ArrayField(models.CharField(max_length=255), null=True)
    status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_action_status",
        null=True,
    )
    context = models.ForeignKey(
        "encounters.EncounterEpisodeOfCareReference",
        on_delete=models.CASCADE,
        related_name="contract_term_action_context",
        null=True,
    )
    context_link_id = ArrayField(models.CharField(max_length=255), null=True)
    occurrence_date_time = models.DateTimeField(null=True)
    occurrence_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="contract_term_action_occurrence_period",
        null=True,
    )
    occurrence_timing = models.ForeignKey(
        Timing,
        related_name="contract_term_action_occurrence_timing",
        on_delete=models.CASCADE,
        null=True,
    )
    requester = models.ManyToManyField(
        ContractTermActionRequesterReference,
        related_name="contract_term_action_requester",
        blank=True,
    )
    requester_link_id = ArrayField(models.CharField(max_length=255), null=True)
    performer_type = models.ManyToManyField(
        CodeableConcept,
        related_name="contract_term_action_performer_type",
        blank=True,
    )
    performer_role = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer_role",
        null=True,
    )
    performer = models.ForeignKey(
        ContractTermActionPerformerReference,
        on_delete=models.CASCADE,
        related_name="contract_term_action_performer",
        null=True,
    )
    performer_link_id = ArrayField(models.CharField(max_length=255), null=True)
    reason = models.ManyToManyField(
        ContractTermActionReasonCodeableReference,
        related_name="contract_term_action_reason",
        blank=True,
    )
    reason_link_id = ArrayField(models.CharField(max_length=255), null=True)
    note = models.ManyToManyField(
        Annotation, related_name="contract_term_action_note", blank=True
    )
    security_label_number = ArrayField(models.IntegerField(), null=True)


class ContractTerm(TimeStampedModel):
    """Contract Term model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="contract_term_identifier",
        null=True,
        on_delete=models.CASCADE,
    )
    issued = models.DateTimeField(null=True)
    applies = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="contract_term_applies",
        null=True,
    )
    topic_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_topic_codeable_concept",
        null=True,
    )
    topic_reference = models.ForeignKey(
        Reference,
        on_delete=models.CASCADE,
        related_name="contract_term_topic_reference",
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_type",
        null=True,
    )
    sub_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_term_sub_type",
        null=True,
    )

    text = models.TextField(null=True)
    security_label = models.ManyToManyField(
        ContractTermSecurityLabel,
        related_name="contract_term_security_label",
        blank=True,
    )

    offer = models.ForeignKey(
        ContractTermOffer,
        on_delete=models.CASCADE,
        related_name="contract_term_offer",
        null=True,
    )

    asset = models.ManyToManyField(
        ContractTermAsset,
        related_name="contract_term_asset",
        blank=True,
    )
    action = models.ManyToManyField(
        ContractTermAction,
        related_name="contract_term_action",
        blank=True,
    )
    group = models.ManyToManyField(
        "self",
        blank=True,
    )


class ContractSignerPartyReference(BaseReference):
    """Contract Signer Party Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_signer_party_reference_identifier",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="contract_signer_party_reference_organization",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="contract_signer_party_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="contract_signer_party_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="contract_signer_party_reference_practitioner_role",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="contract_signer_party_reference_related_person",
        null=True,
    )


class ContractSigner(TimeStampedModel):
    """Contract Signer model."""

    type = models.ForeignKey(
        Coding,
        on_delete=models.CASCADE,
        related_name="contract_signer_type",
        null=True,
    )
    party = models.ForeignKey(
        ContractSignerPartyReference,
        on_delete=models.CASCADE,
        related_name="contract_signer_party",
        null=True,
    )
    signature = models.ManyToManyField(
        Signature,
        related_name="contract_signer_signature",
        blank=True,
    )


class ContractFriendlyContentReferenceReference(BaseReference):
    """Contract Friendly Content Reference Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_friendly_content_reference_reference_identifier",
        null=True,
    )
    # composition = models.ForeignKey(
    #     "compositions.Composition",
    #     on_delete=models.CASCADE,
    #     related_name="contract_friendly_content_reference_reference_composition",
    #     null=True,
    # )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.CASCADE,
        related_name="contract_friendly_content_reference_reference_document_reference",
        null=True,
    )
    # questionnaire_response = models.ForeignKey(
    #     "questionnaireresponses.QuestionnaireResponse",
    #     on_delete=models.CASCADE,
    #     related_name="contract_friendly_content_reference_reference_questionnaire_response",
    #     null=True,
    # )


class ContractLegalContentReferenceReference(BaseReference):
    """Contract Legal Content Reference Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_legal_content_reference_reference_identifier",
        null=True,
    )
    # composition = models.ForeignKey(
    #     "compositions.Composition",
    #     on_delete=models.CASCADE,
    #     related_name="contract_legal_content_reference_reference_composition",
    #     null=True,
    # )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.CASCADE,
        related_name="contract_legal_content_reference_reference_document_reference",
        null=True,
    )
    # questionnaire_response = models.ForeignKey(
    #     "questionnaireresponses.QuestionnaireResponse",
    #     on_delete=models.CASCADE,
    #     related_name="contract_legal_content_reference_reference_questionnaire_response",
    #     null=True,
    # )


class ContractFriendly(TimeStampedModel):
    """Contract Friendly model."""

    content_attachment = models.ForeignKey(
        Attachment,
        on_delete=models.CASCADE,
        related_name="contract_friendly_content_attachment",
        null=True,
    )
    content_reference = models.ForeignKey(
        ContractFriendlyContentReferenceReference,
        on_delete=models.CASCADE,
        related_name="contract_friendly_content_reference",
        null=True,
    )


class ContractLegal(TimeStampedModel):
    """Contract Legal model."""

    content_attachment = models.ForeignKey(
        Attachment,
        on_delete=models.CASCADE,
        related_name="contract_legal_content_attachment",
        null=True,
    )
    content_reference = models.ForeignKey(
        ContractLegalContentReferenceReference,
        on_delete=models.CASCADE,
        related_name="contract_legal_content_reference",
        null=True,
    )


class ContractRule(TimeStampedModel):
    """Contract rule model."""

    content_attachment = models.ForeignKey(
        Attachment,
        on_delete=models.CASCADE,
        related_name="contract_rule_content_attachment",
        null=True,
    )
    # content_reference = models.ForeignKey(
    #     "documentreferences.DocumentReferenceReference",
    #     on_delete=models.CASCADE,
    #     related_name="contract_rule_content_reference",
    #     null=True,
    # )


class ContractLegallyBindingReferenceReference(BaseReference):
    """Contract Legally Binding Reference Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_legally_binding_reference_reference_identifier",
        null=True,
    )
    # composition = models.ForeignKey(
    #     "compositions.Composition",
    #     on_delete=models.CASCADE,
    #     related_name="contract_legally_binding_reference_reference_composition",
    #     null=True,
    # )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.CASCADE,
        related_name="contract_legally_binding_reference_reference_document_reference",
        null=True,
    )
    # questionnaire_response = models.ForeignKey(
    #     "questionnaireresponses.QuestionnaireResponse",
    #     on_delete=models.CASCADE,
    #     related_name="contract_legally_binding_reference_reference_questionnaire_response",
    #     null=True,
    # )
    contract = models.ForeignKey(
        "Contract",
        on_delete=models.CASCADE,
        related_name="contract_legally_binding_reference_reference_contract",
        null=True,
    )


class Contract(TimeStampedModel):
    """Contract model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="contract_identifier",
        blank=True,
    )
    url = models.URLField(null=True)
    version = models.CharField(max_length=255, null=True)
    status = models.CharField(
        max_length=255, null=True, choices=choices.ContractStatusChoices.choices
    )
    legal_state = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_legal_state",
        null=True,
    )
    instantiates_canonical = models.ForeignKey(
        "ContractReference",
        on_delete=models.CASCADE,
        related_name="contract_instantiates_canonical",
        null=True,
    )
    instantiates_uri = models.URLField(null=True)
    content_derivative = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_content_derivative",
        null=True,
    )
    issued = models.DateTimeField(null=True)
    applies = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="contract_applies",
        null=True,
    )
    expiration_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_expiration_type",
        null=True,
    )
    subject = models.ManyToManyField(
        ContractSubjectReference,
        related_name="contract_subject",
        blank=True,
    )
    authority = models.ManyToManyField(
        OrganizationReference,
        related_name="contract_authority",
        blank=True,
    )
    domain = models.ManyToManyField(
        "locations.LocationReference",
        related_name="contract_domain",
        blank=True,
    )
    site = models.ManyToManyField(
        "locations.LocationReference",
        related_name="contract_site",
        blank=True,
    )
    name = models.CharField(max_length=255, null=True)
    title = models.CharField(max_length=255, null=True)
    subtitle = models.CharField(max_length=255, null=True)
    alias = ArrayField(models.CharField(max_length=255), null=True)
    author = models.ForeignKey(
        ContractAuthorReference,
        on_delete=models.CASCADE,
        related_name="contract_author",
        null=True,
    )
    scope = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_scope",
        null=True,
    )
    topic_codeable_concept = models.ManyToManyField(
        CodeableConcept,
        related_name="contract_topic_codeable_concept",
        blank=True,
    )
    topic_reference = models.ManyToManyField(
        Reference,
        related_name="contract_topic_reference",
        blank=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="contract_type",
        null=True,
    )
    sub_type = models.ManyToManyField(
        CodeableConcept,
        related_name="contract_sub_type",
        blank=True,
    )
    content_definition = models.ForeignKey(
        ContractContentDefinition,
        on_delete=models.CASCADE,
        related_name="contract_content_definition",
        null=True,
    )
    term = models.ManyToManyField(
        ContractTerm,
        related_name="contract_term",
        blank=True,
    )
    supporting_info = models.ManyToManyField(
        Reference,
        related_name="contract_supporting_info",
        blank=True,
    )
    relevant_history = models.ManyToManyField(
        "provenances.ProvenanceReference",
        related_name="contract_relevant_history",
        blank=True,
    )
    signer = models.ManyToManyField(
        ContractSigner,
        related_name="contract_signer",
        blank=True,
    )
    friendly = models.ManyToManyField(
        ContractFriendly,
        related_name="contract_friendly",
        blank=True,
    )
    legal = models.ManyToManyField(
        ContractLegal,
        related_name="contract_legal",
        blank=True,
    )
    rule = models.ManyToManyField(
        ContractRule,
        related_name="contract_rule",
        blank=True,
    )
    legally_binding_attachment = models.ForeignKey(
        Attachment,
        on_delete=models.CASCADE,
        related_name="contract_legally_binding_attachment",
        null=True,
    )
    legally_binding_reference = models.ForeignKey(
        ContractLegallyBindingReferenceReference,
        on_delete=models.CASCADE,
        related_name="contract_legally_binding_reference",
        null=True,
    )


class ContractReference(BaseReference):
    """Contract reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="contract_reference_identifier",
        null=True,
    )
    contract = models.ForeignKey(
        Contract,
        on_delete=models.CASCADE,
        related_name="contract_reference_contract",
        null=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('careplans', '0003_initial'),
        ('devices', '0001_initial'),
        ('encounters', '0002_initial'),
        ('medications', '0001_initial'),
        ('provenances', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='MedicationAdministrationDosage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('text', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationAdministrationPartOf',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationAdministrationPerformer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationAdministrationPerformerActorCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationAdministrationPerformerActorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationAdministrationReasonCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationAdministrationReasonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationAdministration',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('in-progress', 'In Progress'), ('not-done', 'Not Done'), ('on-hold', 'On Hold'), ('completed', 'Completed'), ('entered-in-error', 'Entered in Error'), ('stopped', 'Stopped'), ('unknown', 'Unknown')], max_length=200, null=True)),
                ('occurrence_date_time', models.DateTimeField(null=True)),
                ('recorded', models.DateTimeField(null=True)),
                ('is_sub_potent', models.BooleanField(null=True)),
                ('based_on', models.ManyToManyField(blank=True, related_name='medication_administration_based_on', to='careplans.careplanreference')),
                ('category', models.ManyToManyField(blank=True, related_name='medication_administration_category', to='base.codeableconcept')),
                ('device', models.ManyToManyField(blank=True, related_name='medication_administration_device', to='devices.devicecodeablereference')),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_encounter', to='encounters.encounterreference')),
                ('event_history', models.ManyToManyField(blank=True, related_name='medication_administration_event_history', to='provenances.provenancereference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='medication_administration_identifier', to='base.identifier')),
                ('medication', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_medication_codeable_reference', to='medications.medicationcodeablereference')),
                ('note', models.ManyToManyField(blank=True, related_name='medication_administration_note', to='base.annotation')),
                ('occurrence_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_occurrence_period', to='base.period')),
                ('occurrence_timing', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_administration_occurrence_timing', to='base.timing')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('devicedefinitions', '0001_initial'),
        ('devicemetrics', '0002_initial'),
        ('devices', '0001_initial'),
        ('organizations', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='GroupMember',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('inactive', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GroupMemberEntityReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GroupReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GroupCharacteristic',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_uri', models.URLField(null=True)),
                ('exclude', models.BooleanField(null=True)),
                ('description', models.TextField(null=True)),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_code', to='base.codeableconcept')),
                ('determined_by_expression', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_determined_by_expression', to='base.expression')),
                ('duration_duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_duration_duration', to='base.quantity')),
                ('duration_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_duration_range', to='base.range')),
                ('instances_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_instances_quantity', to='base.quantity')),
                ('instances_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_instances_range', to='base.range')),
                ('method', models.ManyToManyField(blank=True, related_name='group_characteristic_method', to='base.codeableconcept')),
                ('offset', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_offset', to='base.codeableconcept')),
                ('period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_period', to='base.period')),
                ('timing', models.ManyToManyField(blank=True, related_name='group_characteristic_timing', to='base.relativetime')),
                ('value_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_value_code', to='base.codeableconcept')),
                ('value_expression', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_value_expression', to='base.expression')),
                ('value_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_value_quantity', to='base.quantity')),
                ('value_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_value_range', to='base.range')),
                ('value_reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_value_reference', to='base.reference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('uri', models.URLField(null=True)),
                ('version', models.CharField(max_length=255, null=True)),
                ('version_algorithm_string', models.CharField(max_length=255, null=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('title', models.CharField(max_length=255, null=True)),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('active', 'Active'), ('retired', 'Retired'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('experimental', models.BooleanField(default=False)),
                ('date', models.DateTimeField(null=True)),
                ('publisher', models.TextField(null=True)),
                ('description', models.TextField(null=True)),
                ('purpose', models.TextField(null=True)),
                ('copyright', models.TextField(null=True)),
                ('copyright_label', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(choices=[('person', 'Person'), ('animal', 'Animal'), ('practitioner', 'Practitioner'), ('device', 'Device'), ('care-team', 'Care Team'), ('healthcare-service', 'Healthcare Service'), ('location', 'Location'), ('organization', 'Organization')], max_length=255, null=True)),
                ('membership', models.CharField(choices=[('definitional', 'Definitional'), ('conceptual', 'Conceptual'), ('enumerated', 'Enumerated')], max_length=255, null=True)),
                ('quantity', models.IntegerField(null=True)),
                ('combination_method', models.CharField(choices=[('all-of', 'All Of'), ('any-of', 'Any Of'), ('at-least', 'At Least'), ('at-most', 'At Most'), ('except-subset', 'Except Subset')], max_length=255, null=True)),
                ('combination_threshold', models.IntegerField(null=True)),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_code', to='base.codeableconcept')),
                ('contact', models.ManyToManyField(blank=True, related_name='group_contact', to='base.contactdetail')),
                ('identifier', models.ManyToManyField(blank=True, related_name='groups_identifier', to='base.identifier')),
                ('use_context', models.ManyToManyField(blank=True, related_name='group_use_context', to='base.usagecontext')),
                ('version_algorithm_coding', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_version_algorithm_coding', to='base.coding')),
                ('characteristic', models.ManyToManyField(blank=True, related_name='group_characteristic', to='groups.groupcharacteristic')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GroupCharacteristicDeterminedByReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_determined_by_reference_device', to='devices.device')),
                ('device_definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='devicedefinitions.devicedefinition')),
                ('device_metric', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_determined_by_reference_device_metric', to='devicemetrics.devicemetric')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_determined_by_reference_identifier', to='base.identifier')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='groupcharacteristic',
            name='determined_by_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_characteristic_determined_by_reference', to='groups.groupcharacteristicdeterminedbyreference'),
        ),
        migrations.CreateModel(
            name='GroupManagingEntityReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_managing_entity_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='group_managing_entity_reference_organization', to='organizations.organization')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""molecular definition model choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class MolecularDefinitionStatusChoices(TextChoices):
    """molecular definition status choices."""

    AA = "aa", _("aa")
    DNA = "dna", _("dna")
    MA = "ma", _("ma")



"""Goals urls."""

from django.urls import path

from . import views

app_name = "goals"

urlpatterns = [
    path("goals/", views.GoalListView.as_view(), name="list_view"),
    path("goals/<int:pk>/", views.GoalDetailView.as_view(), name="detail_view"),
]



"""Endpoints models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    Availability,
    BaseReference,
    CodeableConcept,
    ContactPoint,
    Identifier,
    OrganizationReference,
    Period,
    TimeStampedModel,
)

from . import choices


class EndpointPayload(TimeStampedModel):
    """Endpoint Payload model."""

    type = models.ManyToManyField(
        CodeableConcept, related_name="endpoint_payload_type", blank=True
    )
    mime_type = ArrayField(models.CharField(max_length=255), null=True)
    # TODO: fix this
    # profile_canonical = models.CharField(max_length=255, null=True)
    profile_uri = models.CharField(max_length=255, null=True)


class Endpoint(TimeStampedModel):
    """Endpoint model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="endpoint_identifier", blank=True
    )
    status = models.CharField(
        max_length=255, choices=choices.EndpointStatusChoices.choices, null=True
    )
    connection_type = models.ManyToManyField(
        CodeableConcept, related_name="endpoint_connection_type", blank=True
    )
    name = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)
    environmental_type = models.ManyToManyField(
        CodeableConcept, related_name="endpoint_environmental_type", blank=True
    )
    managing_organization = models.ForeignKey(
        OrganizationReference,
        on_delete=models.DO_NOTHING,
        related_name="endpoint_managing_organization",
        null=True,
    )
    contact = models.ManyToManyField(
        ContactPoint, related_name="endpoint_contact", blank=True
    )
    availability = models.ForeignKey(
        Availability,
        on_delete=models.DO_NOTHING,
        related_name="endpoint_availability",
        null=True,
    )
    period = models.ForeignKey(
        Period, on_delete=models.DO_NOTHING, related_name="endpoint_period", null=True
    )
    payload = models.ManyToManyField(
        EndpointPayload, related_name="endpoint_payload", blank=True
    )
    address = models.CharField(max_length=255, null=True)
    header = ArrayField(models.CharField(max_length=255), null=True)


class EndpointReference(BaseReference):
    """Endpoint Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="endpoint_reference_identifier",
        null=True,
    )
    endpoint = models.ForeignKey(
        "Endpoint",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="endpoint_reference",
    )



"""Nutrition Products models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    Attachment,
    BaseReference,
    CodeableConcept,
    Identifier,
    OrganizationReference,
    Quantity,
    Ratio,
    TimeStampedModel,
)

from . import choices


class NutritionProductNutrients(TimeStampedModel):
    """Nutrition Product Nutrients model."""

    item = models.ForeignKey(
        "substances.SubstanceCodeableReference",
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_nutrients_item",
        null=True,
    )
    amount_ratio = models.ForeignKey(
        Ratio,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_nutrients_amount_ratio",
        null=True,
    )
    amount_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_nutrients_amount_quantity",
        null=True,
    )


class NutritionProductCharacteristic(TimeStampedModel):
    """Nutrition Product Characteristic model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_characteristic_type",
        null=True,
    )
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_characteristic_value_codeable_concept",
        null=True,
    )
    value_string = models.CharField(max_length=255, null=True)
    value_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_characteristic_value_quantity",
        null=True,
    )
    value_boolean = models.BooleanField(null=True)
    value_base64_binary = models.TextField(null=True)
    value_attachment = models.ForeignKey(
        Attachment,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_characteristic_value_attachment",
        null=True,
    )


class NutritionProductInstance(TimeStampedModel):
    """Nutrition Product Instance model."""

    quantity = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_instance_quantity",
        null=True,
    )
    identifier = models.ManyToManyField(
        Identifier, related_name="nutrition_product_instance_identifier", blank=True
    )
    name = models.CharField(max_length=255, null=True)
    lot_number = models.CharField(max_length=255, null=True)
    expiry = models.DateTimeField(null=True)
    use_by = models.DateTimeField(null=True)
    biological_source_event = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_instance_biological_source_event",
        null=True,
    )


class NutritionProductIngredient(TimeStampedModel):
    """Nutrition Product Ingredient model."""

    item = models.ForeignKey(
        "NutritionProductCodeableReference",
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_ingredient_item",
        null=True,
    )
    amount_ratio = models.ForeignKey(
        Ratio,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_ingredient_amount_ratio",
        null=True,
    )
    amount_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_ingredient_amount_quantity",
        null=True,
    )
    allergen = models.BooleanField(null=True)


class NutritionProductReference(BaseReference):
    """Nutrition Product Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_reference_identifier",
        null=True,
    )
    nutrition_product = models.ForeignKey(
        "NutritionProduct",
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_reference_product",
        null=True,
    )


class NutritionProductCodeableReference(TimeStampedModel):
    """Nutrition Product Codeable Reference model."""

    reference = models.ForeignKey(
        NutritionProductReference,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_codeable_reference_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_codeable_reference_concept",
        null=True,
    )


class NutritionProduct(TimeStampedModel):
    """Nutrition Product model."""

    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_code",
        null=True,
    )
    status = models.CharField(
        max_length=255, choices=choices.NutritionProductStatusChoices.choices, null=True
    )
    category = models.ManyToManyField(
        CodeableConcept, related_name="nutrition_product_category", blank=True
    )

    manufacturer = models.ManyToManyField(
        OrganizationReference,
        related_name="nutrition_product_manufacturer",
        blank=True,
    )
    nutrient = models.ManyToManyField(
        NutritionProductNutrients, related_name="nutrition_product_nutrient", blank=True
    )
    ingredient_summary = models.TextField(null=True)
    ingredient = models.ManyToManyField(
        NutritionProductIngredient,
        related_name="nutrition_product_ingredient",
        blank=True,
    )
    energy = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_product_energy",
        null=True,
    )
    characteristic = models.ManyToManyField(
        NutritionProductCharacteristic,
        related_name="nutrition_product_characteristic",
        blank=True,
    )
    instance = models.ManyToManyField(
        NutritionProductInstance, related_name="nutrition_product_instance", blank=True
    )
    note = models.ManyToManyField(
        Annotation, related_name="nutrition_product_note", blank=True
    )



"""Group models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Coding,
    ContactDetail,
    Expression,
    Identifier,
    Period,
    Quantity,
    Range,
    Reference,
    RelativeTime,
    TimeStampedModel,
    UsageContext,
)

from . import choices


class GroupCharacteristicDeterminedByReference(BaseReference):
    """Group characteristic determined by reference model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="group_characteristic_determined_by_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_characteristic_determined_by_reference_device",
    )
    device_metric = models.ForeignKey(
        "devicemetrics.DeviceMetric",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_characteristic_determined_by_reference_device_metric",
    )
    device_definition = models.ForeignKey(
        "devicedefinitions.DeviceDefinition", on_delete=models.CASCADE, null=True
    )


class GroupCharacteristic(TimeStampedModel):
    """Group characteristic model."""

    code = models.ForeignKey(
        CodeableConcept,
        related_name="group_characteristic_code",
        on_delete=models.CASCADE,
        null=True,
    )
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        related_name="group_characteristic_value_code",
        on_delete=models.CASCADE,
        null=True,
    )
    value_boolean = models.BooleanField(null=True)
    value_quantity = models.ForeignKey(
        Quantity,
        related_name="group_characteristic_value_quantity",
        on_delete=models.CASCADE,
        null=True,
    )
    value_range = models.ForeignKey(
        Range,
        related_name="group_characteristic_value_range",
        on_delete=models.CASCADE,
        null=True,
    )
    value_reference = models.ForeignKey(
        Reference,
        related_name="group_characteristic_value_reference",
        on_delete=models.CASCADE,
        null=True,
    )
    value_uri = models.URLField(null=True)
    value_expression = models.ForeignKey(
        Expression,
        related_name="group_characteristic_value_expression",
        on_delete=models.CASCADE,
        null=True,
    )
    exclude = models.BooleanField(null=True)
    description = models.TextField(null=True)
    method = models.ManyToManyField(
        CodeableConcept, related_name="group_characteristic_method", blank=True
    )
    determined_by_reference = models.ForeignKey(
        GroupCharacteristicDeterminedByReference,
        on_delete=models.CASCADE,
        null=True,
        related_name="group_characteristic_determined_by_reference",
    )
    determined_by_expression = models.ForeignKey(
        Expression,
        on_delete=models.CASCADE,
        null=True,
        related_name="group_characteristic_determined_by_expression",
    )
    offset = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        null=True,
        related_name="group_characteristic_offset",
    )
    instances_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.CASCADE,
        null=True,
        related_name="group_characteristic_instances_quantity",
    )
    instances_range = models.ForeignKey(
        Range,
        on_delete=models.CASCADE,
        null=True,
        related_name="group_characteristic_instances_range",
    )
    duration_duration = models.ForeignKey(
        Quantity,
        on_delete=models.CASCADE,
        null=True,
        related_name="group_characteristic_duration_duration",
    )
    duration_range = models.ForeignKey(
        Range,
        on_delete=models.CASCADE,
        null=True,
        related_name="group_characteristic_duration_range",
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        null=True,
        related_name="group_characteristic_period",
    )
    timing = models.ManyToManyField(
        RelativeTime, related_name="group_characteristic_timing", blank=True
    )


class GroupMemberEntityReference(BaseReference):
    """Group member entity reference model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="group_member_entity_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_member_entity_reference_care_team",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_member_entity_reference_device",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_member_entity_reference_group",
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthCareService",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_member_entity_reference_healthcare_service",
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_member_entity_reference_location",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_member_entity_reference_organization",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_member_entity_reference_patient",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_member_entity_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_member_entity_reference_practitioner_role",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_member_entity_reference_related_person",
    )
    specimen = models.ForeignKey(
        "specimens.Specimen",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_member_entity_reference_specimen",
    )


class GroupMember(TimeStampedModel):
    """Group member model."""

    entity = models.ForeignKey(
        GroupMemberEntityReference,
        related_name="group_member_entity",
        on_delete=models.CASCADE,
        null=True,
    )
    involvement = models.ManyToManyField(
        CodeableConcept, related_name="group_member_involvement", blank=True
    )
    period = models.ForeignKey(
        Period, related_name="group_member_period", on_delete=models.CASCADE, null=True
    )
    inactive = models.BooleanField(default=False)


class GroupManagingEntityReference(BaseReference):
    """Group managing entity reference model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="group_managing_entity_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_managing_entity_reference_organization",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_managing_entity_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_managing_entity_reference_practitioner_role",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        null=True,
        related_name="group_managing_entity_reference_related_person",
    )


class Group(TimeStampedModel):
    """Group model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="groups_identifier", blank=True
    )
    uri = models.URLField(null=True)
    version = models.CharField(max_length=255, null=True)
    version_algorithm_string = models.CharField(max_length=255, null=True)
    version_algorithm_coding = models.ForeignKey(
        Coding,
        on_delete=models.CASCADE,
        null=True,
        related_name="group_version_algorithm_coding",
    )
    name = models.CharField(max_length=255, null=True)
    title = models.CharField(max_length=255, null=True)
    status = models.CharField(
        max_length=255, null=True, choices=choices.GroupStatus.choices
    )
    experimental = models.BooleanField(default=False)
    date = models.DateTimeField(null=True)
    publisher = models.TextField(null=True)
    contact = models.ManyToManyField(
        ContactDetail, related_name="group_contact", blank=True
    )
    description = models.TextField(null=True)
    use_context = models.ManyToManyField(
        UsageContext, related_name="group_use_context", blank=True
    )
    purpose = models.TextField(null=True)
    copyright = models.TextField(null=True)
    copyright_label = models.CharField(max_length=255, null=True)
    type = models.CharField(
        max_length=255, null=True, choices=choices.GroupTypeChoices.choices
    )
    membership = models.CharField(
        max_length=255, null=True, choices=choices.GroupMembershipChoices.choices
    )
    code = models.ForeignKey(
        CodeableConcept, related_name="group_code", on_delete=models.CASCADE, null=True
    )
    quantity = models.IntegerField(null=True)
    managing_entity = models.ForeignKey(
        GroupManagingEntityReference,
        related_name="group_managing_entity",
        on_delete=models.CASCADE,
        null=True,
    )
    combination_method = models.CharField(
        max_length=255, null=True, choices=choices.GroupCombinationMethodChoices.choices
    )
    combination_threshold = models.IntegerField(null=True)
    characteristic = models.ManyToManyField(
        GroupCharacteristic, related_name="group_characteristic", blank=True
    )
    member = models.ManyToManyField(
        GroupMember, related_name="group_member", blank=True
    )


class GroupReference(BaseReference):
    """Group reference."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="group_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    group = models.ForeignKey(
        Group, related_name="group_reference_group", on_delete=models.CASCADE, null=True
    )



"""Claim responses model."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    Address,
    Attachment,
    BaseReference,
    CodeableConcept,
    Identifier,
    Money,
    OrganizationReference,
    Period,
    Quantity,
    SimpleQuantity,
    TimeStampedModel,
)

from . import choices


class ClaimResponseRequestorReference(BaseReference):
    """Claim response requestor model."""  # codespell:ignore requestor

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="claim_response_requestor_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="claim_response_requestor_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="claim_response_requestor_practitioner_role",
        null=True,
    )
    organization = models.ForeignKey(
        OrganizationReference,
        on_delete=models.CASCADE,
        related_name="claim_response_requestor_organization",
        null=True,
    )


class ClaimResponseEvent(TimeStampedModel):
    """Claim response event model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_event_type",
        null=True,
    )
    when_date_time = models.DateTimeField(null=True)
    when_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="claim_response_event_when_period",
        null=True,
    )


class ClaimResponseItemReviewOutcome(TimeStampedModel):
    """Claim response item review outcome model."""

    decision = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_item_review_outcome_decision",
        null=True,
    )
    reason = models.ManyToManyField(
        CodeableConcept,
        related_name="claim_response_item_review_outcome_reason",
        blank=True,
    )
    pre_auth_ref = models.CharField(max_length=255, null=True)
    pre_auth_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="claim_response_item_review_outcome_pre_auth_period",
        null=True,
    )


class ClaimResponseItemAdjudication(TimeStampedModel):
    """Claim response item adjudication model."""

    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_item_adjudication_category",
        null=True,
    )
    reason = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_item_adjudication_reason",
        null=True,
    )
    amount = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_item_adjudication_amount",
        null=True,
    )
    quantity = models.ForeignKey(
        Quantity,
        on_delete=models.CASCADE,
        related_name="claim_response_item_adjudication_quantity",
        null=True,
    )


class ClaimResponseItemDetailSubDetail(TimeStampedModel):
    """Claim response item detail sub detail model."""

    sub_detail_sequence = models.PositiveIntegerField()
    trace_number = models.ManyToManyField(
        Identifier,
        related_name="claim_response_item_detail_sub_detail_trace_number",
        blank=True,
    )
    note_number = ArrayField(models.PositiveIntegerField(), null=True)
    review_outcome = models.ForeignKey(
        ClaimResponseItemReviewOutcome,
        on_delete=models.CASCADE,
        related_name="claim_response_item_detail_sub_detail_review_outcome",
        null=True,
    )
    adjudication = models.ManyToManyField(
        ClaimResponseItemAdjudication,
        related_name="claim_response_item_detail_sub_detail_adjudication",
        blank=True,
    )


class ClaimResponseItemDetail(TimeStampedModel):
    """Claim response item detail model."""

    detail_sequence = models.PositiveIntegerField()
    trace_number = models.ManyToManyField(
        Identifier,
        related_name="claim_response_item_detail_trace_number",
        blank=True,
    )
    note_number = ArrayField(models.PositiveIntegerField(), null=True)
    review_outcome = models.ForeignKey(
        ClaimResponseItemReviewOutcome,
        on_delete=models.CASCADE,
        related_name="claim_response_item_detail_review_outcome",
        null=True,
    )
    adjudication = models.ManyToManyField(
        ClaimResponseItemAdjudication,
        related_name="claim_response_item_detail_adjudication",
        blank=True,
    )
    sub_detail = models.ManyToManyField(
        ClaimResponseItemDetailSubDetail,
        related_name="claim_response_item_detail_sub_detail",
        blank=True,
    )


class ClaimResponseItem(TimeStampedModel):
    """Claim response item model."""

    item_sequence = models.PositiveIntegerField()
    trace_number = models.ManyToManyField(
        Identifier,
        related_name="claim_response_item_trace_number",
        blank=True,
    )
    note_number = ArrayField(models.PositiveIntegerField(), null=True)
    review_outcome = models.ForeignKey(
        ClaimResponseItemReviewOutcome,
        on_delete=models.CASCADE,
        related_name="claim_response_item_review_outcome",
        null=True,
    )
    adjudication = models.ManyToManyField(
        ClaimResponseItemAdjudication,
        related_name="claim_response_item_adjudication",
        blank=True,
    )
    detail = models.ManyToManyField(
        ClaimResponseItemDetail,
        related_name="claim_response_item_detail",
        blank=True,
    )


class ClaimResponseAddItemProviderReference(BaseReference):
    """Claim response add item provider reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_provider_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_provider_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_provider_reference_practitioner_role",
        null=True,
    )
    organization = models.ForeignKey(
        OrganizationReference,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_provider_reference_organization",
        null=True,
    )


class ClaimResponseAddItemRequestReference(BaseReference):
    """Claim response add item request reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_request_reference_identifier",
        null=True,
    )
    device_request = models.ForeignKey(
        "devicerequests.DeviceRequest",
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_request_reference_device_request",
        null=True,
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_request_reference_medication_request",
        null=True,
    )
    nutrition_order = models.ForeignKey(
        "nutritionorders.NutritionOrder",
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_request_reference_nutrition_order",
        null=True,
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_request_reference_service_request",
        null=True,
    )
    supply_request = models.ForeignKey(
        "supplyrequests.SupplyRequest",
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_request_reference_supply_request",
        null=True,
    )
    vision_prescription = models.ForeignKey(
        "visionprescriptions.VisionPrescription",
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_request_reference_vision_prescription",
        null=True,
    )


class ClaimResponseAddItemBodySite(TimeStampedModel):
    """Claim response add item body site model."""

    site = models.ManyToManyField(
        "bodystructures.BodyStructureCodeableReference",
        related_name="claim_response_add_item_body_site_body_site",
        blank=True,
    )
    sub_site = models.ManyToManyField(
        CodeableConcept,
        related_name="claim_response_add_item_body_site_sub_site",
        blank=True,
    )


class ClaimResponseAddItemDetailSubDetail(TimeStampedModel):
    """Claim response add item detail sub detail model."""

    trace_number = models.ManyToManyField(
        Identifier,
        related_name="claim_response_add_item_detail_sub_detail_trace_number",
        blank=True,
    )
    revenue = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_sub_detail_revenue",
        null=True,
    )
    product_or_service = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_sub_detail_product_or_service",
        null=True,
    )
    product_or_service_end = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_sub_detail_product_or_service_end",
        null=True,
    )
    modifier = models.ManyToManyField(
        CodeableConcept,
        related_name="claim_response_add_item_detail_sub_detail_modifier",
        blank=True,
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_sub_detail_quantity",
        null=True,
    )
    unit_price = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_sub_detail_unit_price",
        null=True,
    )
    factor = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    tax = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_sub_detail_tax",
        null=True,
    )
    net = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_sub_detail_net",
        null=True,
    )
    note_number = ArrayField(models.PositiveIntegerField(), null=True)
    review_outcome = models.ForeignKey(
        ClaimResponseItemReviewOutcome,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_sub_detail_review_outcome",
        null=True,
    )
    adjudication = models.ManyToManyField(
        ClaimResponseItemAdjudication,
        related_name="claim_response_add_item_detail_sub_detail_adjudication",
        blank=True,
    )


class ClaimResponseAddItemDetail(TimeStampedModel):
    """Claim response add item detail model."""

    trace_number = models.ManyToManyField(
        Identifier,
        related_name="claim_response_add_item_detail_trace_number",
        blank=True,
    )
    revenue = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_revenue",
        null=True,
    )
    product_or_service = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_product_or_service",
        null=True,
    )
    product_or_service_end = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_product_or_service_end",
        null=True,
    )
    modifier = models.ManyToManyField(
        CodeableConcept,
        related_name="claim_response_add_item_detail_modifier",
        blank=True,
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_quantity",
        null=True,
    )
    unit_price = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_unit_price",
        null=True,
    )
    factor = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    tax = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_tax",
        null=True,
    )
    net = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_net",
        null=True,
    )
    note_number = ArrayField(models.PositiveIntegerField(), null=True)
    review_outcome = models.ForeignKey(
        ClaimResponseItemReviewOutcome,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_detail_review_outcome",
        null=True,
    )
    adjudication = models.ManyToManyField(
        ClaimResponseItemAdjudication,
        related_name="claim_response_add_item_detail_adjudication",
        blank=True,
    )
    sub_detail = models.ManyToManyField(
        ClaimResponseAddItemDetailSubDetail,
        related_name="claim_response_add_item_detail_sub_detail",
        blank=True,
    )


class ClaimResponseAddItem(TimeStampedModel):
    """Claim response add item model."""

    item_sequence = ArrayField(models.PositiveIntegerField(), null=True)
    detail_sequence = ArrayField(models.PositiveIntegerField(), null=True)
    sub_detail_sequence = ArrayField(models.PositiveIntegerField(), null=True)
    trace_number = models.ManyToManyField(
        Identifier,
        related_name="claim_response_add_item_trace_number",
        blank=True,
    )
    provider = models.ManyToManyField(
        ClaimResponseAddItemProviderReference,
        related_name="claim_response_add_item_provider",
        blank=True,
    )
    revenue = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_revenue",
        null=True,
    )
    product_or_service = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_product_or_service",
        null=True,
    )
    provider_or_service_end = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_provider_or_service_end",
        null=True,
    )
    request = models.ManyToManyField(
        ClaimResponseAddItemRequestReference,
        related_name="claim_response_add_item_request",
        blank=True,
    )
    modifier = models.ManyToManyField(
        CodeableConcept,
        related_name="claim_response_add_item_modifier",
        blank=True,
    )
    program_code = models.ManyToManyField(
        CodeableConcept,
        related_name="claim_response_add_item_program_code",
        blank=True,
    )
    serviced_date = models.DateField(null=True)
    serviced_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_serviced_period",
        null=True,
    )
    location_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_location_codeable_concept",
        null=True,
    )
    location_address = models.ForeignKey(
        Address,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_location_address",
        null=True,
    )
    location_reference = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_location_reference",
        null=True,
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_quantity",
        null=True,
    )
    unit_price = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_unit_price",
        null=True,
    )
    factor = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    tax = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_tax",
        null=True,
    )
    net = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_net",
        null=True,
    )
    body_site = models.ManyToManyField(
        ClaimResponseAddItemBodySite,
        related_name="claim_response_add_item_body_site",
        blank=True,
    )
    note_number = ArrayField(models.PositiveIntegerField(), null=True)
    review_outcome = models.ForeignKey(
        ClaimResponseItemReviewOutcome,
        on_delete=models.CASCADE,
        related_name="claim_response_add_item_review_outcome",
        null=True,
    )
    adjudication = models.ManyToManyField(
        ClaimResponseItemAdjudication,
        related_name="claim_response_add_item_adjudication",
        blank=True,
    )
    detail = models.ManyToManyField(
        ClaimResponseAddItemDetail,
        related_name="claim_response_add_item_detail",
        blank=True,
    )


class ClaimResponseTotal(TimeStampedModel):
    """Claim response total model."""

    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_total_category",
        null=True,
    )
    amount = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_total_amount",
        null=True,
    )


class ClaimResponsePayment(TimeStampedModel):
    """Claim response payment model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_payment_type",
        null=True,
    )
    adjustment = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_payment_adjustment",
        null=True,
    )
    adjustment_reason = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_payment_adjustment_reason",
        null=True,
    )
    date = models.DateField(null=True)
    amount = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="claim_response_payment_amount",
        null=True,
    )
    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="claim_response_payment_identifier",
        null=True,
    )


class ClaimResponseProcessNote(TimeStampedModel):
    """Claim response process note model."""

    number = models.PositiveIntegerField(null=True)
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_process_note_type",
        null=True,
    )
    text = models.TextField(null=True)
    language = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_process_note_language",
        null=True,
    )


class ClaimResponseInsurance(TimeStampedModel):
    """Claim response insurance model."""

    sequence = models.PositiveIntegerField()
    focal = models.BooleanField()
    coverage = models.ForeignKey(
        "coverages.CoverageReference",
        on_delete=models.CASCADE,
        related_name="claim_response_insurance_coverage",
        null=True,
    )
    business_arrangement = models.CharField(max_length=255, null=True)
    claim_response = models.ForeignKey(
        "ClaimResponseReference",
        on_delete=models.CASCADE,
        related_name="claim_response_insurance_claim_response",
        null=True,
    )


class ClaimResponseError(TimeStampedModel):
    """Claim response error model."""

    item_sequence = models.PositiveIntegerField(null=True)
    detail_sequence = models.PositiveIntegerField(null=True)
    sub_detail_sequence = models.PositiveIntegerField(null=True)
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_error_code",
        null=True,
    )
    expression = ArrayField(models.CharField(max_length=255), null=True)


class ClaimResponse(TimeStampedModel):
    """Claim response model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="claim_response_identifier", blank=True
    )
    trace_number = models.ManyToManyField(
        Identifier, related_name="claim_response_trace_number", blank=True
    )
    status = models.CharField(
        max_length=255,
        choices=choices.ClaimResponseStatus.choices,
        default=choices.ClaimResponseStatus.ACTIVE,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_type",
        null=True,
    )
    sub_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_sub_type",
        null=True,
    )
    use = models.CharField(
        max_length=255,
        choices=choices.ClaimResponseUse.choices,
        default=choices.ClaimResponseUse.CLAIM,
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.CASCADE,
        related_name="claim_response_patient",
        null=True,
    )
    created = models.DateTimeField(null=True)
    insurer = models.ForeignKey(
        OrganizationReference,
        on_delete=models.CASCADE,
        related_name="claim_response_insurer",
        null=True,
    )
    requestor = models.ForeignKey(  # codespell:ignore requestor
        ClaimResponseRequestorReference,
        on_delete=models.CASCADE,
        related_name="claim_response_requestor",  # codespell:ignore requestor
        null=True,
    )
    request = models.ForeignKey(
        "claims.ClaimReference",
        on_delete=models.CASCADE,
        related_name="claim_response_request",
        null=True,
    )
    outcome = models.CharField(
        max_length=255, choices=choices.ClaimResponseOutcome.choices, null=True
    )
    decision = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_decision",
        null=True,
    )
    disposition = models.TextField(null=True)
    pre_auth_ref = models.CharField(max_length=255, null=True)
    pre_auth_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="claim_response_pre_auth_period",
        null=True,
    )
    event = models.ManyToManyField(
        ClaimResponseEvent, related_name="claim_response_event", blank=True
    )
    payee_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_payee_type",
        null=True,
    )
    encounter = models.ManyToManyField(
        "encounters.EncounterReference",
        related_name="claim_response_encounter",
        blank=True,
    )
    diagnosis_related_group = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_diagnosis_related_group",
        null=True,
    )
    item = models.ManyToManyField(
        ClaimResponseItem, related_name="claim_response_item", blank=True
    )
    add_item = models.ManyToManyField(
        ClaimResponseAddItem, related_name="claim_response_add_item", blank=True
    )
    adjudication = models.ManyToManyField(
        ClaimResponseItemAdjudication,
        related_name="claim_response_adjudication",
        blank=True,
    )
    total = models.ManyToManyField(
        ClaimResponseTotal, related_name="claim_response_total", blank=True
    )
    payment = models.ForeignKey(
        ClaimResponsePayment,
        on_delete=models.CASCADE,
        related_name="claim_response_payment",
        null=True,
    )
    funds_reserve = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_funds_reserve",
        null=True,
    )
    form_code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="claim_response_form_code",
        null=True,
    )
    form = models.ForeignKey(
        Attachment,
        on_delete=models.CASCADE,
        related_name="claim_response_form",
        null=True,
    )
    process_note = models.ManyToManyField(
        ClaimResponseProcessNote, related_name="claim_response_process_note", blank=True
    )
    # communication_request = models.ManyToManyField(
    #     "communicationrequests.CommunicationRequestReference",
    #     related_name="claim_response_communication_request",
    #     blank=True,
    # )
    insurance = models.ManyToManyField(
        ClaimResponseInsurance, related_name="claim_response_insurance", blank=True
    )
    error = models.ManyToManyField(
        ClaimResponseError, related_name="claim_response_error", blank=True
    )


class ClaimResponseReference(BaseReference):
    """Claim response reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="claim_response_reference_identifier",
        null=True,
    )
    claim_response = models.ForeignKey(
        ClaimResponse,
        on_delete=models.CASCADE,
        related_name="claim_response_reference_claim_response",
        null=True,
    )



"""medication administrations models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Reference,
    TimeStampedModel,
)
from dfhir.medicationadministrations.choices import MedicationAdministrationStatus


class MedicationAdministrationPartOf(BaseReference):
    """medication administration part of model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        null=True,
        related_name="medication_administration_part_of_identifier",
    )
    medication_administraton = models.ForeignKey(
        "MedicationAdministration",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_part_of_medication_administration",
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_part_of_procedure",
    )
    medication_dispense = models.ForeignKey(
        "medicationdispenses.MedicationDispense",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_part_of_medication_dispense",
    )


class MedicationAdministrationPerformerActorReference(BaseReference):
    """medication administration performer actor model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_performer_actor_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_performer_actor_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_performer_actor_practitioner_role",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_performer_actor_patient",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_performer_actor_related_person",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_performer_actor_device",
    )


class MedicationAdministrationPerformerActorCodeableReference(TimeStampedModel):
    """medication administration performer actor codeable reference model."""

    concept = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.SET_NULL,
        related_name="medication_administration_performer_actor_codeable_reference_concept",
    )
    reference = models.ForeignKey(
        MedicationAdministrationPerformerActorReference,
        null=True,
        on_delete=models.SET_NULL,
        related_name="medication_administration_performer_actor_codeable_reference_reference",
    )


class MedicationAdministrationPerformer(TimeStampedModel):
    """medication administration performer model."""

    function = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_performer_function",
    )
    actor = models.ForeignKey(
        MedicationAdministrationPerformerActorCodeableReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_performer_actor",
    )


class MedicationAdministrationReasonReference(BaseReference):
    """medication administration reason model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_reason_reference_identifier",
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_reason_reference_condition",
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_reason_reference_observation",
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_reason_reference_diagnostic_report",
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_reason_reference_procedure",
    )


class MedicationAdministrationReasonCodeableReference(TimeStampedModel):
    """medication administration reason codeable reference model."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_reason_codeable_reference_codeable_concept",
    )
    reference = models.ForeignKey(
        MedicationAdministrationReasonReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_reason_codeable_reference_reference",
    )


class MedicationAdministrationDosage(TimeStampedModel):
    """medication administration dosage model."""

    text = models.TextField(null=True)
    site = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_dosage_site",
    )
    route = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_dosage_route",
    )
    method = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_dosage_method",
    )
    dose = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_dosage_dose",
    )
    rate_ratio = models.ForeignKey(
        "base.Ratio",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_dosage_rate_ratio",
    )
    rate_quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_dosage_rate_quantity",
    )


class MedicationAdministration(TimeStampedModel):
    """medication administration model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="medication_administration_identifier", blank=True
    )
    based_on = models.ManyToManyField(
        "careplans.CarePlanReference",
        related_name="medication_administration_based_on",
        blank=True,
    )
    part_of = models.ManyToManyField(
        MedicationAdministrationPartOf,
        related_name="medication_administration_part_of",
        blank=True,
    )
    status = models.CharField(
        max_length=200, choices=MedicationAdministrationStatus.choices, null=True
    )
    status_reason = models.ManyToManyField(
        CodeableConcept,
        related_name="medication_administration_status_reason",
        blank=True,
    )
    category = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="medication_administration_category",
    )
    medication = models.ForeignKey(
        "medications.MedicationCodeableReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_medication_codeable_reference",
    )
    subject = models.ForeignKey(
        "patients.PatientGroupReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_subject",
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_encounter",
    )
    supporting_information = models.ManyToManyField(
        Reference,
        related_name="medication_administration_supporting_information",
    )
    occurrence_date_time = models.DateTimeField(null=True)
    occurrence_period = models.ForeignKey(
        "base.Period",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_occurrence_period",
    )
    occurrence_timing = models.ForeignKey(
        "base.Timing",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_occurrence_timing",
    )
    recorded = models.DateTimeField(null=True)
    is_sub_potent = models.BooleanField(null=True)
    sub_potent_reason = models.ManyToManyField(
        CodeableConcept,
        related_name="medication_administration_sub_potent_reason",
        blank=True,
    )
    performer = models.ManyToManyField(
        MedicationAdministrationPerformer,
        related_name="medication_administration_performer",
        blank=True,
    )
    reason = models.ManyToManyField(
        MedicationAdministrationReasonCodeableReference,
        related_name="medication_administration_reason_codeable_reference",
        blank=True,
    )
    request = models.ForeignKey(
        "medicationrequests.MedicationRequestReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_request",
    )
    device = models.ManyToManyField(
        "devices.DeviceCodeableReference",
        related_name="medication_administration_device",
        blank=True,
    )
    note = models.ManyToManyField(
        "base.Annotation",
        related_name="medication_administration_note",
        blank=True,
    )
    dosage = models.ForeignKey(
        MedicationAdministrationDosage,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_administration_dosage",
    )
    event_history = models.ManyToManyField(
        "provenances.ProvenanceReference",
        related_name="medication_administration_event_history",
        blank=True,
    )



# Generated by Django 5.2 on 2025-05-05 14:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('supplydeliveries', '0001_initial'),
        ('supplyrequests', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='supplydelivery',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='supply_delivery_based_on', to='supplyrequests.supplyrequestreference'),
        ),
    ]
